package javax.management.monitor;

import javax.management.ObjectName;
import java.util.Iterator;
import java.util.logging.Level;
import com.sun.jmx.defaults.JmxProperties;
import javax.management.MBeanNotificationInfo;

public class GaugeMonitor extends Monitor implements GaugeMonitorMBean
{
    private Number highThreshold;
    private Number lowThreshold;
    private boolean notifyHigh;
    private boolean notifyLow;
    private boolean differenceMode;
    private static final String[] types;
    private static final MBeanNotificationInfo[] notifsInfo;
    private static final int RISING = 0;
    private static final int FALLING = 1;
    private static final int RISING_OR_FALLING = 2;
    
    public GaugeMonitor() {
        this.highThreshold = GaugeMonitor.INTEGER_ZERO;
        this.lowThreshold = GaugeMonitor.INTEGER_ZERO;
        this.notifyHigh = false;
        this.notifyLow = false;
        this.differenceMode = false;
    }
    
    @Override
    public synchronized void start() {
        if (this.isActive()) {
            JmxProperties.MONITOR_LOGGER.logp(Level.FINER, GaugeMonitor.class.getName(), "start", "the monitor is already active");
            return;
        }
        Iterator<Monitor.ObservedObject> iterator = this.observedObjects.iterator();
        while (iterator.hasNext()) {
            GaugeMonitor.GaugeMonitorObservedObject gaugeMonitorObservedObject = (GaugeMonitor.GaugeMonitorObservedObject)(Monitor.ObservedObject)iterator.next();
            gaugeMonitorObservedObject.setStatus(2);
            gaugeMonitorObservedObject.setPreviousScanGauge(null);
        }
        this.doStart();
        return;
    }
    
    @Override
    public synchronized void stop() {
        this.doStop();
    }
    
    @Override
    public synchronized Number getDerivedGauge(ObjectName objectName) {
        return (Number)super.getDerivedGauge(objectName);
    }
    
    @Override
    public synchronized long getDerivedGaugeTimeStamp(ObjectName objectName) {
        return super.getDerivedGaugeTimeStamp(objectName);
    }
    
    @Deprecated
    @Override
    public synchronized Number getDerivedGauge() {
        if (this.observedObjects.isEmpty()) {
            return null;
        }
        return (Number)((Monitor.ObservedObject)this.observedObjects.get(0)).getDerivedGauge();
    }
    
    @Deprecated
    @Override
    public synchronized long getDerivedGaugeTimeStamp() {
        if (this.observedObjects.isEmpty()) {
            return 0L;
        }
        return ((Monitor.ObservedObject)this.observedObjects.get(0)).getDerivedGaugeTimeStamp();
    }
    
    @Override
    public synchronized Number getHighThreshold() {
        return this.highThreshold;
    }
    
    @Override
    public synchronized Number getLowThreshold() {
        return this.lowThreshold;
    }
    
    @Override
    public synchronized void setThresholds(Number n, Number n2) throws IllegalArgumentException {
        if (n == null || n2 == null) {
            throw new IllegalArgumentException("Null threshold value");
        }
        if (n.getClass() != n2.getClass()) {
            throw new IllegalArgumentException("Different type threshold values");
        }
        if (this.isFirstStrictlyGreaterThanLast(n2, n, n.getClass().getName())) {
            throw new IllegalArgumentException("High threshold less than low threshold");
        }
        if (this.highThreshold.equals(n) && this.lowThreshold.equals(n2)) {
            return;
        }
        this.highThreshold = n;
        this.lowThreshold = n2;
        int n3 = 0;
        for (Monitor.ObservedObject observedObject : this.observedObjects) {
            this.resetAlreadyNotified(observedObject, n3++, 16);
            ((GaugeMonitor.GaugeMonitorObservedObject)observedObject).setStatus(2);
        }
        return;
    }
    
    @Override
    public synchronized boolean getNotifyHigh() {
        return this.notifyHigh;
    }
    
    @Override
    public synchronized void setNotifyHigh(boolean notifyHigh) {
        if (this.notifyHigh == notifyHigh) {
            return;
        }
        this.notifyHigh = notifyHigh;
        return;
    }
    
    @Override
    public synchronized boolean getNotifyLow() {
        return this.notifyLow;
    }
    
    @Override
    public synchronized void setNotifyLow(boolean notifyLow) {
        if (this.notifyLow == notifyLow) {
            return;
        }
        this.notifyLow = notifyLow;
        return;
    }
    
    @Override
    public synchronized boolean getDifferenceMode() {
        return this.differenceMode;
    }
    
    @Override
    public synchronized void setDifferenceMode(boolean differenceMode) {
        if (this.differenceMode == differenceMode) {
            return;
        }
        this.differenceMode = differenceMode;
        Iterator<Monitor.ObservedObject> iterator = this.observedObjects.iterator();
        while (iterator.hasNext()) {
            GaugeMonitor.GaugeMonitorObservedObject gaugeMonitorObservedObject = (GaugeMonitor.GaugeMonitorObservedObject)(Monitor.ObservedObject)iterator.next();
            gaugeMonitorObservedObject.setStatus(2);
            gaugeMonitorObservedObject.setPreviousScanGauge(null);
        }
        return;
    }
    
    @Override
    public MBeanNotificationInfo[] getNotificationInfo() {
        return (MBeanNotificationInfo[])GaugeMonitor.notifsInfo.clone();
    }
    
    private synchronized boolean updateDerivedGauge(Object o, GaugeMonitor.GaugeMonitorObservedObject gaugeMonitorObservedObject) {
        int n;
        if (this.differenceMode) {
            if (gaugeMonitorObservedObject.getPreviousScanGauge() != null) {
                this.setDerivedGaugeWithDifference((Number)o, gaugeMonitorObservedObject);
                n = true;
            }
            else {
                n = false;
            }
            gaugeMonitorObservedObject.setPreviousScanGauge((Number)o);
        }
        else {
            gaugeMonitorObservedObject.setDerivedGauge((Object)o);
            n = true;
        }
        return n;
    }
    
    private synchronized MonitorNotification updateNotifications(GaugeMonitor.GaugeMonitorObservedObject gaugeMonitorObservedObject) {
        MonitorNotification monitorNotification = null;
        if (gaugeMonitorObservedObject.getStatus() == 2) {
            if (this.isFirstGreaterThanLast((Number)gaugeMonitorObservedObject.getDerivedGauge(), this.highThreshold, gaugeMonitorObservedObject.getType())) {
                if (this.notifyHigh) {
                    monitorNotification = new MonitorNotification("jmx.monitor.gauge.high", this, 0L, 0L, "", null, null, null, this.highThreshold);
                }
                gaugeMonitorObservedObject.setStatus(1);
            }
            else if (this.isFirstGreaterThanLast(this.lowThreshold, (Number)gaugeMonitorObservedObject.getDerivedGauge(), gaugeMonitorObservedObject.getType())) {
                if (this.notifyLow) {
                    monitorNotification = new MonitorNotification("jmx.monitor.gauge.low", this, 0L, 0L, "", null, null, null, this.lowThreshold);
                }
                gaugeMonitorObservedObject.setStatus(0);
            }
        }
        else if (gaugeMonitorObservedObject.getStatus() == 0) {
            if (this.isFirstGreaterThanLast((Number)gaugeMonitorObservedObject.getDerivedGauge(), this.highThreshold, gaugeMonitorObservedObject.getType())) {
                if (this.notifyHigh) {
                    monitorNotification = new MonitorNotification("jmx.monitor.gauge.high", this, 0L, 0L, "", null, null, null, this.highThreshold);
                }
                gaugeMonitorObservedObject.setStatus(1);
            }
        }
        else if (gaugeMonitorObservedObject.getStatus() == 1 && this.isFirstGreaterThanLast(this.lowThreshold, (Number)gaugeMonitorObservedObject.getDerivedGauge(), gaugeMonitorObservedObject.getType())) {
            if (this.notifyLow) {
                monitorNotification = new MonitorNotification("jmx.monitor.gauge.low", this, 0L, 0L, "", null, null, null, this.lowThreshold);
            }
            gaugeMonitorObservedObject.setStatus(0);
        }
        return monitorNotification;
    }
    
    private synchronized void setDerivedGaugeWithDifference(Number n, GaugeMonitor.GaugeMonitorObservedObject gaugeMonitorObservedObject) {
        Number previousScanGauge = gaugeMonitorObservedObject.getPreviousScanGauge();
        Integer derivedGauge;
        switch (gaugeMonitorObservedObject.getType()) {
            case INTEGER: {
                derivedGauge = Integer.valueOf((int)n - (int)previousScanGauge);
                break;
            }
            case BYTE: {
                derivedGauge = Byte.valueOf((byte)((byte)n - (byte)previousScanGauge));
                break;
            }
            case SHORT: {
                derivedGauge = Short.valueOf((short)((short)n - (short)previousScanGauge));
                break;
            }
            case LONG: {
                derivedGauge = Long.valueOf((long)n - (long)previousScanGauge);
                break;
            }
            case FLOAT: {
                derivedGauge = Float.valueOf((float)n - (float)previousScanGauge);
                break;
            }
            case DOUBLE: {
                derivedGauge = Double.valueOf((double)n - (double)previousScanGauge);
                break;
            }
            default: {
                JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, GaugeMonitor.class.getName(), "setDerivedGaugeWithDifference", "the threshold type is invalid");
                return;
            }
        }
        gaugeMonitorObservedObject.setDerivedGauge(derivedGauge);
    }
    
    private boolean isFirstGreaterThanLast(Number n, Number n2, Monitor.NumericalType numericalType) {
        switch (numericalType) {
            case INTEGER:
            case BYTE:
            case SHORT:
            case LONG: {
                return n.longValue() >= n2.longValue();
            }
            case FLOAT:
            case DOUBLE: {
                return n.doubleValue() >= n2.doubleValue();
            }
            default: {
                JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, GaugeMonitor.class.getName(), "isFirstGreaterThanLast", "the threshold type is invalid");
                return false;
            }
        }
    }
    
    private boolean isFirstStrictlyGreaterThanLast(Number n, Number n2, String s) {
        if (s.equals("java.lang.Integer") || s.equals("java.lang.Byte") || s.equals("java.lang.Short") || s.equals("java.lang.Long")) {
            return n.longValue() > n2.longValue();
        }
        if (s.equals("java.lang.Float") || s.equals("java.lang.Double")) {
            return n.doubleValue() > n2.doubleValue();
        }
        JmxProperties.MONITOR_LOGGER.logp(Level.FINEST, GaugeMonitor.class.getName(), "isFirstStrictlyGreaterThanLast", "the threshold type is invalid");
        return false;
    }
    
    @Override
    Monitor.ObservedObject createObservedObject(ObjectName objectName) {
        GaugeMonitor.GaugeMonitorObservedObject gaugeMonitorObservedObject = new GaugeMonitorObservedObject(objectName);
        gaugeMonitorObservedObject.setStatus(2);
        gaugeMonitorObservedObject.setPreviousScanGauge(null);
        return gaugeMonitorObservedObject;
    }
    
    @Override
    synchronized boolean isComparableTypeValid(ObjectName objectName, String s, Comparable<?> comparable) {
        GaugeMonitor.GaugeMonitorObservedObject gaugeMonitorObservedObject = (GaugeMonitor.GaugeMonitorObservedObject)this.getObservedObject(objectName);
        if (gaugeMonitorObservedObject == null) {
            return false;
        }
        if ((comparable instanceof Integer)) {
            gaugeMonitorObservedObject.setType(Monitor.NumericalType.INTEGER);
        }
        else if ((comparable instanceof Byte)) {
            gaugeMonitorObservedObject.setType(Monitor.NumericalType.BYTE);
        }
        else if ((comparable instanceof Short)) {
            gaugeMonitorObservedObject.setType(Monitor.NumericalType.SHORT);
        }
        else if ((comparable instanceof Long)) {
            gaugeMonitorObservedObject.setType(Monitor.NumericalType.LONG);
        }
        else if ((comparable instanceof Float)) {
            gaugeMonitorObservedObject.setType(Monitor.NumericalType.FLOAT);
        }
        else if ((comparable instanceof Double)) {
            gaugeMonitorObservedObject.setType(Monitor.NumericalType.DOUBLE);
        }
        else {
            return false;
        }
        return true;
    }
    
    @Override
    synchronized Comparable<?> getDerivedGaugeFromComparable(ObjectName objectName, String s, Comparable<?> comparable) {
        GaugeMonitor.GaugeMonitorObservedObject gaugeMonitorObservedObject = (GaugeMonitor.GaugeMonitorObservedObject)this.getObservedObject(objectName);
        if (gaugeMonitorObservedObject == null) {
            return null;
        }
        gaugeMonitorObservedObject.setDerivedGaugeValid(this.updateDerivedGauge(comparable, gaugeMonitorObservedObject));
        return (Comparable)gaugeMonitorObservedObject.getDerivedGauge();
    }
    
    @Override
    synchronized void onErrorNotification(MonitorNotification monitorNotification) {
        GaugeMonitor.GaugeMonitorObservedObject gaugeMonitorObservedObject = (GaugeMonitor.GaugeMonitorObservedObject)this.getObservedObject(monitorNotification.getObservedObject());
        if (gaugeMonitorObservedObject == null) {
            return;
        }
        gaugeMonitorObservedObject.setStatus(2);
        gaugeMonitorObservedObject.setPreviousScanGauge(null);
        return;
    }
    
    @Override
    synchronized MonitorNotification buildAlarmNotification(ObjectName objectName, String s, Comparable<?> comparable) {
        GaugeMonitor.GaugeMonitorObservedObject gaugeMonitorObservedObject = (GaugeMonitor.GaugeMonitorObservedObject)this.getObservedObject(objectName);
        if (gaugeMonitorObservedObject == null) {
            return null;
        }
        if (gaugeMonitorObservedObject.getDerivedGaugeValid()) {
            return this.updateNotifications(gaugeMonitorObservedObject);
        }
        return null;
    }
    
    @Override
    synchronized boolean isThresholdTypeValid(ObjectName objectName, String s, Comparable<?> comparable) {
        GaugeMonitor.GaugeMonitorObservedObject gaugeMonitorObservedObject = (GaugeMonitor.GaugeMonitorObservedObject)this.getObservedObject(objectName);
        if (gaugeMonitorObservedObject == null) {
            return false;
        }
        Class<Object> classForType = Monitor.classForType(gaugeMonitorObservedObject.getType());
        return Monitor.isValidForType(this.highThreshold, classForType) && Monitor.isValidForType(this.lowThreshold, classForType);
    }
    
    static {
        types = new String[] { "jmx.monitor.error.runtime", "jmx.monitor.error.mbean", "jmx.monitor.error.attribute", "jmx.monitor.error.type", "jmx.monitor.error.threshold", "jmx.monitor.gauge.high", "jmx.monitor.gauge.low" };
        notifsInfo = new MBeanNotificationInfo[] { new MBeanNotificationInfo(GaugeMonitor.types, "javax.management.monitor.MonitorNotification", "Notifications sent by the GaugeMonitor MBean") };
    }
    
    static class GaugeMonitorObservedObject extends Monitor.ObservedObject
    {
        private boolean derivedGaugeValid;
        private Monitor.NumericalType type;
        private Number previousScanGauge;
        private int status;
        
        public GaugeMonitorObservedObject(ObjectName objectName) {
            super(objectName);
        }
        
        public final synchronized boolean getDerivedGaugeValid() {
            return this.derivedGaugeValid;
        }
        
        public final synchronized void setDerivedGaugeValid(boolean derivedGaugeValid) {
            this.derivedGaugeValid = derivedGaugeValid;
        }
        
        public final synchronized Monitor.NumericalType getType() {
            return this.type;
        }
        
        public final synchronized void setType(Monitor.NumericalType type) {
            this.type = type;
        }
        
        public final synchronized Number getPreviousScanGauge() {
            return this.previousScanGauge;
        }
        
        public final synchronized void setPreviousScanGauge(Number previousScanGauge) {
            this.previousScanGauge = previousScanGauge;
        }
        
        public final synchronized int getStatus() {
            return this.status;
        }
        
        public final synchronized void setStatus(int status) {
            this.status = status;
        }
    }
}
